#!/usr/bin/env python3
import asyncio
import inspect
import time
import sys
import os
import yaml
from aiohttp import ClientSession
from datetime import datetime


try:
    from skodaconnect import Connection
except ModuleNotFoundError as e:
    print(f"Unable to import library: {e}")
    sys.exit(1)


class skoda_class:
    def __init__(self, config=None, config_file=".skoda.cfg", verbose=False):
        self.conn = None
        self.loop = asyncio.get_event_loop()
        self.state = {}
        if config is not None:
            self.state = config
        else:
            self.config_file = config_file
            self.get_config()
        if verbose:
            self.state["verbose"] = True
        else:
            self.state["verbose"] = False

    def get_config(self):
        try:
            with open(self.config_file, "r") as file:
                self.state = yaml.load(file, Loader=yaml.FullLoader)
        except Exception as e:
            print(f"can't open config file: {self.config_file}  {e}")

    async def login_and_get_vehicles(self):
        """Main method."""
        if self.state.get("verbose"):
            print("function: login_and_get_vehicles")
        session = ClientSession(headers={"Connection": "keep-alive"})
        if self.state.get("verbose"):
            print(
                f"Initiating new session to Skoda Connect with {self.state['username']} as username"
            )
        connection = Connection(
            session, self.state["username"], self.state["password"], False
        )
        self.conn = connection
        if self.state.get("verbose"):
            print("Attempting to login to the Skoda Connect service")
            print(datetime.now())
        if await connection.doLogin():
            if self.state.get("verbose"):
                print("Login success!")
                print(datetime.now())
                print("Fetching vehicles associated with account.")
            await connection.get_vehicles()
            await connection.update_all()
            return (session, connection.vehicles)
        return (session, None)

    async def async_get_vehicle(self, vehicle_vin):
        if self.state.get("verbose"):
            print("function: async_get_vehicle")
        (session, vehicles) = await self.login_and_get_vehicles()
        if vehicles == None:
            print("no vehicle found to set temp.")
            await session.close()
            return (None, None)
        if vehicle_vin == None:
            if len(vehicles) > 1:
                print("multiple vehicles found, please select one using vin.")
                await session.close()
                return (None, None)
            return (session, vehicles[0])
        for vehicle in vehicles:
            if vehicle_vin == vehicle.vin:
                return (session, vehicle)
        await session.close()
        return (None, None)

    async def async_get_instruments(self, vehicle_vin):
        """Async airco."""
        if self.state.get("verbose"):
            print("function: async_get_instruments")
        (session, vehicle) = await self.async_get_vehicle(vehicle_vin)
        if vehicle == None:
            print("no vehicle found.")
            await session.close()
            return None
        dashboard = vehicle.dashboard()
        instruments = dashboard.instruments
        await session.close()
        return instruments


    async def async_print_instruments(self, vehicle_vin):
        # Battery level
        if self.state.get("verbose"):
            print("function: aync_print_instruments")
        instruments = await self.async_get_instruments(vehicle_vin)
        if instruments == None:
            print('no instruments')
        for i in instruments:
            print(f"{i.name} = {i.state}")
        return None

    def print_instruments(self, vehicle_vin=None):
        return self.loop.run_until_complete(
            self.async_print_instruments(vehicle_vin)
        )


    async def async_get_battery_level(self, vehicle_vin):
        # Battery level
        if self.state.get("verbose"):
            print("function: async_get_battery_level")
        instruments = await self.async_get_instruments(vehicle_vin)
        if instruments == None:
            return None
        for i in instruments:
            if i.name == "Battery level":
                return int(i.state)
        return None


    def get_battery_level(self, vehicle_vin=None):
        if self.state.get("verbose"):
            print("function: get_battery_level")
        return self.loop.run_until_complete(
            self.async_get_battery_level(vehicle_vin)
        )

    def get_instruments(self, vehicle_vin=None):
        if self.state.get("verbose"):
            print("function: get_instruments")
        result = self.loop.run_until_complete(
            self.async_get_instruments(vehicle_vin)
        )
        instruments={}
        for i in result:
            instruments[i.name] = i.state
        return instruments

    async def async_list_cars(self):
        """Async airco."""
        if self.state.get("verbose"):
            print("function: async_list_cars")
        (session, vehicles) = await self.login_and_get_vehicles()
        if vehicles == None:
            print("no vehicle found.")
            await session.close()
            return False
        for vehicle in vehicles:
            print(f"VIN: {vehicle.vin}\t: {vehicle.model}")
        await session.close()


    def list_cars(self):
        if self.state.get("verbose"):
            print("function: list_cars") 
        print(f"listing cars")
        return self.loop.run_until_complete(self.async_list_cars())



""" Load config. """

import appdirs
from .__version__ import __app_name__, __author_name__, __version__
import os
import yaml

def get_config_dir_name():
    config_dir = appdirs.user_config_dir(__app_name__, __author_name__)
    return config_dir


def get_config_file_name(name="config.yaml"):
    config_dir = get_config_dir_name()
    config_file = os.path.join(config_dir, name)
    return config_file


def get_config(name="config.yaml", full_path=None):
    if full_path is None or full_path == "":
        config_file = get_config_file_name(name=name)
    else:
        config_file = full_path
    try:
        with open(config_file) as f:
            yaml_config = yaml.safe_load(f)
    except FileNotFoundError:
        print(f"can't open config file: {config_file}")
        exit(-1)
    return yaml_config


def get_data_dir_name():
        data_dir = appdirs.user_data_dir(__app_name__, __author_name__)
        return data_dir


def get_data_file_name(name="data.yaml"):
        data_dir = get_data_dir_name()
        data_file = os.path.join(data_dir, name)
        return data_file




import time
import requests

# get car types:
# curl https://api.iternio.com/1/tlm/get_carmodels_list | jq . | grep skoda
#      "Skoda;ENYAQ iV;50 (alpha)": "skoda:enyaq:21:52:meb"
#      "Skoda;ENYAQ iV;60 (alpha)": "skoda:enyaq:21:58:meb"
#      "Skoda;ENYAQ iV;80 (alpha)": "skoda:enyaq:21:77:meb"
#      "Skoda;ENYAQ iV;80x (alpha)": "skoda:enyaq:21:77:meb:x"
#      "Skoda;ENYAQ iV;RS (alpha)": "skoda:enyaq:21:77:meb:rs"
#      "Skoda;ENYAQ iV;portline (alpha)": "skoda:enyaq:21:77:meb:sportline"


class abrp_class:
    def __init__(self, token, api_key='c626070c-1c8d-4003-bc76-6de223b44679', url='https://api.iternio.com/1', car_type='skoda:enyaq:21:77:meb', debug_output=None):
        self.token = token
        self.api_key = api_key
        self.url = url
        self.car_type = car_type
        self.debug_output = debug_output
        self.fail_output = print

    def debug(self, str):
        if self.debug_output != None:
            self.debug_output(str)

    def fail(self, str):
        if self.fail_output != None:
            self.fail_output(str)

    def create_data(self, input_data):
        output_data = {}
        output_data['utc'] = int(time.time())
        output_data['soc'] = input_data['Battery level'] 
        output_data['car_model'] = self.car_type
        output_data['is_charging'] = input_data['Charging']
        output_data['est_battery_range'] = input_data['Electric range']
        output_data['power'] = int(input_data['Charging Power'] / -1000)
        self.debug(str(output_data))
        return output_data

    def send(self, output_data):
        try:
            headers = {"Authorization": f"APIKEY {self.api_key}"}
            data = {"tlm": output_data}
            result = requests.post(f"{self.url}/tlm/send?token={self.token}", headers=headers, json=data)
        except Exception as e:
            self.fail(f'post issue: {str(e)}')
        return result


    def send_data(self, input_data):
        output_data = self.create_data(input_data)
        return self.send(output_data)

    def get_car_types(self):
        try:
            result = requests.get(f"{self.url}/tlm/get_carmodels_list")
        except Exception as e:
            self.fail(f'get issue: {str(e)}')
        # TODO check result status code
        return result.json()



import paho.mqtt.client as mqtt

client = None
output = None

def on_connect(client, userdata, flags, rc):
    if rc == 0:
        if output is not None:
            output("mqtt: Connected successfully")
    else:
        if output is not None:
            output("mqtt: Connect returned result code: " + str(rc))
        else:
            print("mqtt: Connect returned result code: " + str(rc))


def on_publish(client, userdata, result):             #create function for callback
    #print("mqtt: data published \n")
    pass


def mqtt_connect(host, port, tls = False, username=None, password=None, info_output=None):
    global client
    global output
    output = info_output
    client = mqtt.Client()
    client.on_publish = on_publish
    client.on_connect = on_connect

    if tls is True:
        client.tls_set(tls_version=mqtt.ssl.PROTOCOL_TLS)

    if password is not None:
        client.username_pw_set(username, password)

    client.connect(host, port)
    client.loop_start()
    return client

def mqtt_publish(client, topic, value):
    result = client.publish(topic, value)


def mqtt_publish_instruments(client, topic, instruments):
    mqtt_publish(client, f"{topic}/battery_level", instruments['Battery level'])
    mqtt_publish(client, f"{topic}/charging", instruments['Charging'])
    mqtt_publish(client, f"{topic}/charging_power", instruments['Charging Power'])
    mqtt_publish(client, f"{topic}/charging_rate", instruments['Charging rate'])
    mqtt_publish(client, f"{topic}/climatisation_target_temperature", instruments['Climatisation target temperature'])
    mqtt_publish(client, f"{topic}/electric_range", instruments['Electric range'])
    mqtt_publish(client, f"{topic}/minimum_charge_level", instruments['Minimum charge level'])





""" Loop. """
from datetime import datetime
from rich.console import Console

console = None

def console_str(my_str):
    # [22:12:14]
    now = datetime.now()
    global console
    if console is None:
        console = Console()

    current_time = now.strftime("%H:%M:%S")
    console.print(f"[cyan]\[{current_time}][/cyan] {my_str}")
    #print(f"{current_time} | {my_str}")


